import { Global, Module } from '@nestjs/common';
import { TypeOrmModule } from '@nestjs/typeorm';
import { DataSource } from 'typeorm';
import { APP_DATASOURCES } from './database.constants';
import { DatabaseConfig, databaseConfig } from '../../config/database.config';
import { typeormTransactionalDataSourceFactory } from './typeorm-data-source-factory';
import { typeormModuleOptionsFactory } from './typeorm-module-options-factory';
import { typeormEntitiesFromImport } from './typeorm-import-entities';

const dataSources: DataSource[] = [];

const typeormModules = [
  TypeOrmModule.forRootAsync({
    name: 'ebitemp-api',
    dataSourceFactory: typeormTransactionalDataSourceFactory('ebitemp-api', dataSources),
    useFactory: async (dbConfig: DatabaseConfig) => {
      // const entities = await import('./path/to/entities');
      const entities = {};

      const config = await typeormModuleOptionsFactory('ebitemp-api', dbConfig, [
        typeormEntitiesFromImport(entities),
      ]);
      return config;
    },
    inject: [databaseConfig.KEY],
  }),
];

const dataSourcesProvider = {
  provide: APP_DATASOURCES,
  useValue: dataSources,
};

@Global()
@Module({
  imports: [...typeormModules],
  providers: [dataSourcesProvider],
  exports: [...typeormModules, dataSourcesProvider],
})
export class AppDatabaseModule {}
